use std::io;
use std::io::Write;
use std::process::exit;

struct InputBuffer {
    buffer: Option<String>,
}

enum StatementType {
    Insert,
    Select,
}

struct Statement {
    statement_type: StatementType,
}

impl Statement {
    fn new(statement_type: StatementType) -> Self {
        Self {
            statement_type
        }
    }

    pub(crate) fn execute(&self) {
        match self.statement_type {
            StatementType::Insert => {
                println!("This is where you do an insert")
            }
            StatementType::Select => {
                println!("This is where you do an select")
            }
        }
    }
}

impl InputBuffer {
    pub fn new() -> Self {
        return Self {
            buffer: None
        };
    }

    pub(crate) fn print_prompt(&mut self) -> &mut Self {
        print!("db > ");
        if let Err(_) = io::stdout().flush() {
            panic!("could not print to stdout")
        }
        self
    }

    pub(crate) fn read_input(&mut self) -> Result<&mut Self, String> {
        self.buffer = Some(String::new());
        match &mut self.buffer {
            Some(input_buffer) => {
                if let Ok(_) = io::stdin().read_line(input_buffer) {
                    Ok(self)
                } else {
                    Err(String::from("could not handle input"))
                }
            }
            _ => { Err(String::from("Could not initialize buffer")) }
        }
    }

    pub(crate) fn parse(&self) {
        match &self.buffer {
            Some(command) => {
                if command.starts_with(".") {
                    Self::handle_meta_statement(command);
                } else {
                    if let Ok(statement) = Self::prepare_statement(command) {
                        statement.execute()
                    } else {
                        println!("could not recognize statement");
                    }
                }
            }
            None => {}
        }
    }

    fn prepare_statement(command: &String) -> Result<Statement, String> {
        if command.starts_with("insert") {
            Ok(Statement::new(StatementType::Insert))
        } else if command.starts_with("select") {
            Ok(Statement::new(StatementType::Select))
        } else {
            Err(String::from("Unrecognized statement"))
        }
    }

    fn handle_meta_statement(command: &String) {
        match command.replace("\n", "").trim() {
            ".exit" => {
                exit(0);
            }
            cmd => {
                println!("Could not handle command: {cmd}")
            }
        }
    }
}


fn main() -> Result<(), String> {
    let mut input_buffer = InputBuffer::new();

    loop {
        input_buffer
            .print_prompt()
            .read_input()?
            .parse()
    }
}
